// Copyright 2008-2018 Yolo Technologies, Inc. All Rights Reserved. https://www.comblockengine.com

#ifndef KBE_DB_INTERFACE_H
#define KBE_DB_INTERFACE_H

#include "common/common.h"
#include "common/singleton.h"
#include "helper/debug_helper.h"
#include "db_interface/entity_table.h"
#include "server/serverconfig.h"

namespace KBEngine { 

namespace thread
{
class ThreadPool;
}

class DBUtil;

/*
	ݿӿ
*/
class DBInterface
{
public:
	enum DB_OP_STATE
	{
		DB_OP_READ,
		DB_OP_WRITE,
	};

	friend class DBUtil;

	DBInterface(const char* name) :
	db_port_(3306),
	db_numConnections_(1),
	lastquery_()
	{
		strncpy(name_, name, MAX_NAME - 1);
		int dbIndex = g_kbeSrvConfig.dbInterfaceName2dbInterfaceIndex(this->name());
		KBE_ASSERT(dbIndex >= 0);
		dbIndex_ = dbIndex;
	};

	virtual ~DBInterface()
	{
	};

	/**
		黷
	*/
	virtual bool checkEnvironment() = 0;
	
	/**
		 Դݽо
		ɹʧ
	*/
	virtual bool checkErrors() = 0;

	/**
		ĳݿ
	*/
	virtual bool attach(const char* databaseName = NULL) = 0;
	virtual bool detach() = 0;

	/**
		ȡݿеı
	*/
	virtual bool getTableNames( std::vector<std::string>& tableNames, const char * pattern) = 0;

	/**
		ȡݿĳеֶ
	*/
	virtual bool getTableItemNames(const char* tableName, std::vector<std::string>& itemNames) = 0;

	/**
		ѯ
	*/
	virtual bool query(const char* cmd, uint32 size, bool printlog = true, MemoryStream * result = NULL) = 0;
	virtual bool query(const std::string& cmd, bool printlog = true, MemoryStream * result = NULL)
	{
		return query(cmd.c_str(), (uint32)cmd.size(), printlog, result);
	}

	/**
		ӿڵ
	*/
	const char* name() const { return name_; }

	/**
		ӿڵ
	*/
	uint16 dbIndex() const { return dbIndex_; }

	/**
		ӿڵ
	*/
	virtual const char* c_str() = 0;

	/** 
		ȡ
	*/
	virtual const char* getstrerror() = 0;

	/** 
		ȡ
	*/
	virtual int getlasterror() = 0;

	/**
		һentity洢
	*/
	virtual EntityTable* createEntityTable(EntityTables* pEntityTables) = 0;

	/** 
		ݿɾentity
	*/
	virtual bool dropEntityTableFromDB(const char* tableName) = 0;

	/** 
		ݿɾentityֶ
	*/
	virtual bool dropEntityTableItemFromDB(const char* tableName, const char* tableItemName) = 0;

	/**
		סӿڲ
	*/
	virtual bool lock() = 0;
	virtual bool unlock() = 0;

	/**
		쳣
	*/
	virtual bool processException(std::exception & e) = 0;

	/**
		ȡһβѯsql
	*/
	virtual const std::string& lastquery() const{ return lastquery_; }

protected:
	char name_[MAX_BUF];									// ݿӿڵ
	char db_type_[MAX_BUF];									// ݿ
	uint32 db_port_;										// ݿĶ˿
	char db_ip_[MAX_IP];									// ݿipַ
	char db_username_[MAX_BUF];								// ݿû
	char db_password_[MAX_BUF * 10];						// ݿ
	char db_name_[MAX_BUF];									// ݿ
	uint16 db_numConnections_;								// ݿ
	std::string lastquery_;									// һβѯ
	uint16 dbIndex_;										// Ӧݿӿ
};

/*
	ݿԪ
*/
class DBUtil : public Singleton<DBUtil>
{
public:
	DBUtil();
	~DBUtil();
	
	static bool initialize();
	static void finalise();
	static bool initializeWatcher();

	static bool initThread(const std::string& dbinterfaceName);
	static bool finiThread(const std::string& dbinterfaceName);

	static DBInterface* createInterface(const std::string& name, bool showinfo = true);
	static const char* accountScriptName();
	static bool initInterface(DBInterface* pdbi);

	static void handleMainTick();

	typedef KBEUnordered_map<std::string, thread::ThreadPool*> DBThreadPoolMap;
	static thread::ThreadPool* pThreadPool(const std::string& name)
	{ 
		DBThreadPoolMap::iterator iter = pThreadPoolMaps_.find(name);
		if (iter != pThreadPoolMaps_.end())
			return iter->second;

		return NULL;
	}

private:
	static DBThreadPoolMap pThreadPoolMaps_;
};

}

#endif // KBE_DB_INTERFACE_H
